@@ -22,6 +22,6 @@ package v1beta2
 const (
 	// DefaultCACertPath defines default location of CA certificate on Windows
 	DefaultCACertPath = "C:/etc/kubernetes/pki/ca.crt"
-	// DefaultUrlScheme defines default socket url prefix
-	DefaultUrlScheme = "npipe"
+	// DefaultContainerRuntimeURLScheme defines default socket url prefix
+	DefaultContainerRuntimeURLScheme = "npipe"
 )
