@@ -150,12 +150,12 @@ func (mounter *Mounter) MountSensitive(source string, target string, fstype stri
 		mklinkSource = mklinkSource + "\\"
 	}
 
-	err := os.Symlink(mklinkSource, target)
+	output, err := exec.Command("cmd", "/c", "mklink", "/D", target, mklinkSource).CombinedOutput()
 	if err != nil {
-		klog.Errorf("symlink failed: %v, source(%q) target(%q)", err, mklinkSource, target)
+		klog.Errorf("mklink failed: %v, source(%q) target(%q) output: %q", err, mklinkSource, target, string(output))
 		return err
 	}
-	klog.V(2).Infof("symlink source(%q) on target(%q) successfully", mklinkSource, target)
+	klog.V(2).Infof("mklink source(%q) on target(%q) successfully, output: %q", mklinkSource, target, string(output))
 
 	return nil
 }
@@ -219,9 +219,8 @@ func removeSMBMapping(remotepath string) (string, error) {
 func (mounter *Mounter) Unmount(target string) error {
 	klog.V(4).Infof("Unmount target (%q)", target)
 	target = NormalizeWindowsPath(target)
-
-	if err := os.Remove(target); err != nil {
-		klog.Errorf("removing directory %s failed: %v", target, err)
+	if output, err := exec.Command("cmd", "/c", "rmdir", target).CombinedOutput(); err != nil {
+		klog.Errorf("rmdir failed: %v, output: %q", err, string(output))
 		return err
 	}
 	return nil
@@ -274,7 +273,7 @@ func (mounter *Mounter) GetMountRefs(pathname string) ([]string, error) {
 	return []string{pathname}, nil
 }
 
-func (mounter *SafeFormatAndMount) formatAndMountSensitive(source string, target string, fstype string, options []string, sensitiveOptions []string) error {
+func (mounter *SafeFormatAndMount) formatAndMountSensitive(source string, target string, fstype string, options []string, sensitiveOptions []string, formatOptions []string) error {
 	// Try to mount the disk
 	klog.V(4).Infof("Attempting to formatAndMount disk: %s %s %s", fstype, source, target)
 
@@ -288,6 +287,10 @@ func (mounter *SafeFormatAndMount) formatAndMountSensitive(source string, target
 		fstype = "NTFS"
 	}
 
+	if len(formatOptions) > 0 {
+		return fmt.Errorf("diskMount: formatOptions are not supported on Windows")
+	}
+
 	cmdString := "Get-Disk -Number $env:source | Where partitionstyle -eq 'raw' | Initialize-Disk -PartitionStyle GPT -PassThru" +
 		" | New-Partition -UseMaximumSize | Format-Volume -FileSystem $env:fstype -Confirm:$false"
 	cmd := mounter.Exec.Command("powershell", "/c", cmdString)
@@ -323,7 +326,7 @@ func listVolumesOnDisk(diskID string) (volumeIDs []string, err error) {
 	cmd.Env = append(os.Environ(), fmt.Sprintf("diskID=%s", diskID))
 	klog.V(8).Infof("Executing command: %q", cmd.String())
 	output, err := cmd.CombinedOutput()
-	klog.V(4).Infof("ListVolumesOnDisk id from %s: %s", diskID, string(output))
+	klog.V(4).Infof("listVolumesOnDisk id from %s: %s", diskID, string(output))
 	if err != nil {
 		return []string{}, fmt.Errorf("error list volumes on disk. cmd: %s, output: %s, error: %v", cmd, string(output), err)
 	}
